<?php
/*
 * timeline-db.php
 * Description: Database functions for the SIMILE Timline Plugin
 * Plugin URI: http://www.freshlabs.de/journal/archives/2006/10/wordpress-plugin-simile-timeline/
 * Author: Tim Isenheim
 * 
	===========================================================================
	SIMILE Timeline for WordPress
	Copyright (C) 2006 Tim Isenheim
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	===========================================================================
*/

/* ---------------------------------------------------------------------------------
 * adds extra column in posts table for start and end dates
 * --------------------------------------------------------------------------------*/
function stl_timeline_add_eventcolumn($column_name){
	global $wpdb;
	if(!stl_timeline_column_exists($column_name)) {
		$wpdb->query($wpdb->prepare("ALTER TABLE $wpdb->posts ADD COLUMN " . $column_name . " datetime NOT NULL DEFAULT '0000-00-00 00:00:00'"));
	}
}

/* ---------------------------------------------------------------------------------
 * removes the database column on uninstalling
 * WARNING: This deletes all end dates set for posts
 * --------------------------------------------------------------------------------*/
function stl_timeline_remove_eventcolumn($column_name) {
	global $wpdb;
	if(stl_timeline_column_exists($column_name)) {
		$wpdb->query($wpdb->prepare("ALTER TABLE $wpdb->posts DROP COLUMN $column_name"));
	}
}

/* ---------------------------------------------------------------------------------
 * Save the start date of an event(post) to the database
 * --------------------------------------------------------------------------------*/
function stl_timeline_update_eventstart_box($pID){
	__stl_timeline_update_event_date($pID, $_POST, 'start');
	__stl_timeline_update_event_date($pID, $_POST, 'latest_start');
}

/* ---------------------------------------------------------------------------------
 * Save the end date of an event(post) to the database
 * --------------------------------------------------------------------------------*/
function stl_timeline_update_eventend_box($pID){
	__stl_timeline_update_event_date($pID, $_POST, 'end');
	__stl_timeline_update_event_date($pID, $_POST, 'earliest_end');
}

/* ---------------------------------------------------------------------------------
 * Save time value of an event(post) to the database
 * used by both, start and end update functions
 * --------------------------------------------------------------------------------*/
function __stl_timeline_update_event_date($pID, $p, $type='start'){
	global $wpdb;
	$column = "stl_timeline_event_".$type;
	$edit = $p['stl_edit_timeline_'.$type.'_date'];
	$reset = $p['stl_reset_timeline_'.$type.'_date'];
	
	// if "Edit timestamp" is selected
	if (isset($edit) && !isset($reset)) {
		$sj = 'stl_jj_'.$type;
		$stl_aa = $p['stl_aa_'.$type];
		$stl_mm = $p['stl_mm_'.$type];
		$stl_jj = $p[$sj];
		$stl_hh = $p['stl_hh_'.$type];
		$stl_mn = $p['stl_mn_'.$type];
		$stl_ss = $p['stl_ss_'.$type];
		$stl_jj = ($stl_jj > 31) ? 31 : $stl_jj;
		$stl_hh = ($stl_hh > 23) ? $stl_hh -24 : $stl_hh;
		$stl_mn = ($stl_mn > 59) ? $stl_mn -60 : $stl_mn;
		$stl_ss = ($stl_ss > 59) ? $stl_ss -60 : $stl_ss;
		$postdata = "$stl_aa-$stl_mm-$stl_jj $stl_hh:$stl_mn:$stl_ss";
		
		$stl_tee = $postdata;
		
		$wpdb->query($wpdb->prepare("UPDATE $wpdb->posts SET $column = '$stl_tee' WHERE ID = $pID"));
	}
	// Reset timestamp to 0
	if (isset ($reset)) {
		$wpdb->query($wpdb->prepare("UPDATE $wpdb->posts SET $column = '0000-00-00 00:00:00' WHERE ID = $pID"));
	}
}

/* ---------------------------------------------------------------------------------
 * stl_get_extreme_event_date
 * Get the date of the very first or last post dependant from the start date
 * --------------------------------------------------------------------------------*/
function stl_get_extreme_event_date($minmax, $type, $format='r', $categories){
	global $wpdb;
	
	$column = 'stl_timeline_event_' . $type;
	
	// try to get the extreme post date from stl-columns
	$date = stl_timeline_query_postdate($minmax, $column, $categories);
	
	if(empty($date)){
		$column = 'post_date';
		// get usual post date when start or end date isn't set
		$date = stl_timeline_query_postdate($minmax, $column, $categories);
	}
	if($format != null){
		$date = adodb_date2($format, $date);
	}
	return $date;
}

/* ---------------------------------------------------------------------------------
 * stl_timeline_query_postdate
 * Get the minimum or maximum date of wpdb->posts.$column in $categories
 * --------------------------------------------------------------------------------*/
function stl_timeline_query_postdate($minmax, $column, $categories){
	global $wpdb;
	
	$relation = "SELECT object_id FROM $wpdb->term_relationships " .
				"INNER JOIN $wpdb->term_taxonomy ON($wpdb->term_relationships.term_taxonomy_id = $wpdb->term_taxonomy.term_taxonomy_id AND	$wpdb->term_taxonomy.term_id IN ($categories) )";
				
	$query = 	"SELECT $minmax($wpdb->posts.$column) from $wpdb->posts INNER JOIN ($relation) t1 ON (t1.object_id=$wpdb->posts.ID) " .
				"WHERE $column != '0000-00-00 00:00:00' AND post_status='publish' AND post_type!='revision'";

	return $wpdb->get_var($query);
}

/* ---------------------------------------------------------------------------------
 * stl_getdate
 * PHP getdate pendant...the dirty way
 * --------------------------------------------------------------------------------*/
function stl_getdate($date){
	$d = explode(' ', $date);
	$date = explode('-', $d[0]);
	$time = explode(':', $d[1]);
	$arr['year'] = $date[0];
	$arr['month'] = $date[1];
	$arr['day'] = $date[2];
	$arr['hour'] = $time[0];
	$arr['minute'] = $time[1];
	$arr['second'] = $time[2];
	return $arr;
}

/* ---------------------------------------------------------------------------------
 * install STL table for Timeline category attributes
 * --------------------------------------------------------------------------------*/
function stl_timeline_add_relationship_table(){
		
	global $wpdb;

	$table_name = $wpdb->prefix . "stl_timeline_terms";
	if($wpdb->get_var("SHOW TABLES LIKE '$table_name'") != $table_name) {

		$sql = "CREATE TABLE " . $table_name . " (
			  id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
			  term_id INT NOT NULL,
			  color VARCHAR( 7 ) NOT NULL DEFAULT '#58A0DC',
			  active INT NOT NULL DEFAULT 0
			);";

		require_once(ABSPATH . 'wp-admin/upgrade-functions.php');
		dbDelta($sql);
		
		// copy all category-terms to the new table
		$sql2 = "SELECT * FROM $wpdb->term_taxonomy WHERE taxonomy = 'category' OR taxonomy = 'link_category'";

		$res = $wpdb->get_results($sql2);

		foreach($res as $term){
			stl_add_term_to_db($term);
		}
   }
}

/* ---------------------------------------------------------------------------------
 * Delete STL table for Timeline category attributes
 * --------------------------------------------------------------------------------*/
function stl_timeline_remove_relationship_table(){
		
	global $wpdb;

	$table_name = $wpdb->prefix . "stl_timeline_terms";
	$wpdb->query($wpdb->prepare("DROP TABLE " . $table_name));
}

/* ---------------------------------------------------------------------------------
 * update STL table for Timeline categories
 * --------------------------------------------------------------------------------*/
function stl_timeline_update_relationship_table(){
	
	global $wpdb;
	require_once(ABSPATH . 'wp-admin/upgrade-functions.php');
		
	// copy diff of term table to timeline-terms
	$table = $wpdb->prefix . "stl_timeline_terms";
	$sql = "SELECT term_id FROM $table ";
	$res = $wpdb->get_results($sql, ARRAY_A);
	
	$terms = get_terms(array('category', 'link_category'));
	
	$cmp = array();
	// copy categories to stl_timeline term relationship table
	foreach($res as $stl_term){
		array_push($cmp, $stl_term['term_id']);
	}
	// loop all available valid terms
	foreach($terms as $term){		
		// add to database if not already exist in the timeline terms 
		if(!in_array($term->term_id, $cmp)){
			stl_add_term_to_db($term);
		}
		else{
			// already exists.. do nothing
		}
	}
}

/* ---------------------------------------------------------------------------------
 * remove category from options if the whole category is deleted
 * --------------------------------------------------------------------------------*/
function stl_delete_cat_from_options($cat_ID) {
	$s = trim(get_option('stl_timelinecategories'));
	// TODO: do some regex here
	$s = str_replace(",$cat_ID", "", $s);
	update_option('stl_timelinecategories', $s);
}

/* ---------------------------------------------------------------------------------
 * remove term from options if the whole category is deleted
 * --------------------------------------------------------------------------------*/
function stl_delete_term_from_db($term_id) {
	global $wpdb;

	$table_name = $wpdb->prefix . "stl_timeline_terms";
	
	$insert = "DELETE FROM " . $table_name . " WHERE term_id= " . $term_id . "";
	$results = $wpdb->query($wpdb->prepare($insert));
}

/* ---------------------------------------------------------------------------------
 * add single term to timeline relationship table
 * --------------------------------------------------------------------------------*/
function stl_add_term_to_db($term) {
	global $wpdb;
	require_once(ABSPATH . 'wp-admin/upgrade-functions.php');
	
	$table_name = $wpdb->prefix . "stl_timeline_terms";

	$insert = "INSERT INTO " . $table_name . " (term_id) " .
		  	  "VALUES ('".$term->term_id."')";
	$results = $wpdb->query($wpdb->prepare($insert));

}

/* ---------------------------------------------------------------------------------
 * get term from timeline relationship table for specific post ID
 * --------------------------------------------------------------------------------*/
function stl_get_timeline_term_for_post($post_id) {
	global $wpdb;
	
	$table_name = $wpdb->prefix . "stl_timeline_terms";
	
	$term_id = $wpdb->get_var("SELECT term_id FROM $wpdb->term_taxonomy " .
			"INNER JOIN $wpdb->term_relationships ON($wpdb->term_relationships.term_taxonomy_id = $wpdb->term_taxonomy.term_taxonomy_id AND
			$wpdb->term_relationships.object_id = $post_id)");

	return stl_get_term($term_id);
}

/* ---------------------------------------------------------------------------------
 * get term from timeline relationship table for specific post ID
 * --------------------------------------------------------------------------------*/
function stl_get_term($term_id) {
	global $wpdb;
	
	$table_name = $wpdb->prefix . "stl_timeline_terms";

	$get = "SELECT * FROM " . $table_name . " WHERE term_id = $term_id";
	$row = $wpdb->get_row($get);
	return $row;
}

/* ---------------------------------------------------------------------------------
 * Save the category active attribute into database
 * --------------------------------------------------------------------------------*/
function stl_timeline_update_catactive($cID, $status){
	global $wpdb;
	//extract($_POST);
	$wpdb->query($wpdb->prepare("UPDATE ".$wpdb->prefix."stl_timeline_terms SET active = '$status' WHERE term_id = $cID"));
}

/* ---------------------------------------------------------------------------------
 * Save the category color into database
 * --------------------------------------------------------------------------------*/
function stl_timeline_update_catcolor($cID, $color){
	global $wpdb;
	//extract($_POST);
	$wpdb->query($wpdb->prepare("UPDATE ".$wpdb->prefix."stl_timeline_terms SET color = '$color' WHERE term_id = $cID"));
}

/* ---------------------------------------------------------------------------------
 * checks if a given column exists in a database table
 * --------------------------------------------------------------------------------*/
function stl_timeline_column_exists($col_name){
	global $wpdb;
	$column_exists = false;
	$q = $wpdb->query($wpdb->prepare("SHOW COLUMNS FROM $wpdb->posts LIKE '$col_name'"));
	if($q == 1){
		$column_exists = true;
	}
	return $column_exists;
}
?>