<?php
/*
Plugin Name:  WP SIMILE Timeline
Plugin URI: http://www.freshlabs.de/journal/archives/2006/10/wordpress-plugin-simile-timeline/
Description: Integrates the <a href="http://simile.mit.edu/timeline/">SIMILE Timeline</a> with WordPress and provides an option interface for the various timeline settings. With this plugin you can display posts from a specific category in the Timeline Widget. Simply include the <strong>[similetimeline]</strong> shortcode in your page or post and specify the category on the <a href="options-general.php?page=timeline.php">admin page</a>.
Author: Tim Isenheim
Author URI: http://www.freshlabs.de/journal
Version: 0.4.6
*/
/*
	SIMILE Timeline for WordPress
    Copyright (C) 2006 Tim Isenheim

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
// TODO: i18n echo for documentation
include('inc/timeline-db.inc.php');	// Timeline database functions
include('inc/adodb-time.inc.php');  // AdoDB Time+Date Library
include('options/options-timeline.php');  // option page

$stl_use_timeline = false;			// initially no timeline is used

@define('STL_TIMELINE_PLUGIN_DATESTRING', '20090912');
@define('STL_TIMELINE_PLUGIN_VERSION', '0.4.6');
@define('STL_TIMELINE_FOLDER', WP_PLUGIN_URL.'/wp-simile-timeline');
@define('STL_TIMELINE_DATA_FOLDER', STL_TIMELINE_FOLDER.'/data');
@define('STL_TIMELINE_API_URL', 'http://static.simile.mit.edu/timeline/api-2.3.0/timeline-api.js?bundle=true');

$currentLocale = get_locale();
if(!empty($currentLocale)) {
	$moFile = dirname(__FILE__) . "/locale/" . $currentLocale . ".mo";
	if(@file_exists($moFile) && is_readable($moFile)) load_textdomain('stl_timeline', $moFile);
}

$stl_timeline_api_vars = array(
	'cats' => '0',
	'id' => 'stl-mytimeline',
	'scriptfile'=>'timeline.js.php',
	'theme'=>'dynamic-theme'
);

/* -----------------------------------------------------------------------------
 * Install the plugin with basic parameters and necessary database tables
 * ---------------------------------------------------------------------------*/
function stl_timeline_init_plugin(){
	stl_timeline_add_eventcolumn("stl_timeline_event_start"); // add database table for start date
	stl_timeline_add_eventcolumn("stl_timeline_event_latest_start"); // add database table for start date
	stl_timeline_add_eventcolumn("stl_timeline_event_end"); // add database table for end date
	stl_timeline_add_eventcolumn("stl_timeline_event_earliest_end"); // add database table for end date
	stl_timeline_add_relationship_table();  // add database table for categories
	stl_upgrade();
}

/* -----------------------------------------------------------------------------
 * Setup all the necessary options
 * ---------------------------------------------------------------------------*/
function stl_upgrade(){
	$stl_timeline_default_options = stl_timeline_get_default_options();
	foreach($stl_timeline_default_options as $option=>$v){
		if(!get_option($option))
			add_option($option, $v);
	}
	update_option('stl_timeline_plugin_version', STL_TIMELINE_PLUGIN_VERSION);
	// update possibly new categories
	stl_timeline_update_relationship_table();
}

/* -----------------------------------------------------------------------------
 * Plugin uninstaller. Removes all plugin related data from database
 * ---------------------------------------------------------------------------*/
function stl_do_plugin_uninstall(){
	$stl_timeline_default_options = stl_timeline_get_default_options();
	// remove entries from option table
	foreach($stl_timeline_default_options as $option=>$v){
		delete_option($option);
	}
	// remove columns in wp_posts
	stl_timeline_remove_eventcolumn("stl_timeline_event_start"); // remove database table for start date
	stl_timeline_remove_eventcolumn("stl_timeline_event_latest_start"); // remove database table for start date
	stl_timeline_remove_eventcolumn("stl_timeline_event_end"); // remove database table for end date
	stl_timeline_remove_eventcolumn("stl_timeline_event_earliest_end"); // remove database table for end date
	// remove stl_timeline_terms table
	stl_timeline_remove_relationship_table();
	deactivate_plugins(array('wp-simile-timeline/timeline.php'));
	wp_redirect('plugins.php?deactivate=true');
	die();
}

/* -----------------------------------------------------------------------------
 * stl_simile_timeline
 * manually output the Timeline markup
 * ---------------------------------------------------------------------------*/
function stl_simile_timeline($cats='', $id='stl-mytimeline', $scriptfile='timeline.js.php', $theme='dark-theme'){
	echo stl_output_timeline_markup(array('cats'=>$cats, 'id'=>$id, 'scriptfile'=>$scriptfile, 'theme'=>$theme));
}

/* -----------------------------------------------------------------------------
 * stl_output_timeline_markup
 * print the necessary timeline markup (called by shortcode API) and Template Tag
 * ---------------------------------------------------------------------------*/
function stl_output_timeline_markup($atts){
	global $stl_timeline_api_vars;
	// make attributes available as variables (see http://codex.wordpress.org/Shortcode_API)
	extract( shortcode_atts( $stl_timeline_api_vars, $atts ) );
	
	$r = '<div id="'.$id.'" class="stl-timeline '.$theme.'"></div>';
	/*
	 * TODO: Move this to the <head> section with the correct filter (wp_print_scripts)
	 * but still with the dynamic attribute array.
	 */
	$script = '<script src="'.STL_TIMELINE_DATA_FOLDER.'/'.$scriptfile.'?' .
	$script .= 'v='.STL_TIMELINE_PLUGIN_DATESTRING.'&amp;id='.$id;
	if(!empty($cats)) $script .= '&amp;cat='.$cats;
	$script .= '" type="text/javascript"></script>';
	$script .= '<script src="'.STL_TIMELINE_FOLDER.'/inc/onload.js" type="text/javascript"></script>';
	return $r . $script;
}

/* -----------------------------------------------------------------------------
 * stl_is_timeline_page
 * loops all page ids given in the timeline options
 * returns true when the current page has a timeline
 * ---------------------------------------------------------------------------*/
function stl_is_timeline_page($stl_currentpageid){
	$stl_istimelinepage = false;
	$stl_timelinepages = get_option('stl_timelinepageids');
	// Set timeline for all pages when the option is 0
	if($stl_timelinepages == 0 || $stl_timelinepages == 'all'){
		$stl_istimelinepage = true;
	}
	else{
		$stl_timelinepagesarray = split(",", $stl_timelinepages);
		foreach($stl_timelinepagesarray as $stl_timelinepageid):
			if($stl_currentpageid == $stl_timelinepageid){
				$stl_istimelinepage = true;
			}
		endforeach;
	}
	return $stl_istimelinepage;
}

/* -----------------------------------------------------------------------------
 * stl_headerJavaScript
 * outputs the JavaScript implementation in the <head> element
 * ONLY when post_id is set in the SIMILE Timeline options
 * ---------------------------------------------------------------------------*/
function stl_headerJavaScriptCSS(){
	global $post, $stl_use_timeline;

	if( stl_is_timeline_page($post->ID) || $stl_use_timeline ):
		// load API from SIMILE server
		echo '<script src="'.STL_TIMELINE_API_URL.'" type="text/javascript"></script>'.
		// include local CSS template for timeline rules
		'<link rel="stylesheet" href="'.WP_PLUGIN_URL.'/wp-simile-timeline/timeline.css" type="text/css" />' . "\n" .
		'<link rel="stylesheet" href="'.WP_PLUGIN_URL.'/wp-simile-timeline/data/timeline.css.php" type="text/css" />' . "\n";
	endif;
}

/* -----------------------------------------------------------------------------
     Outputs necessary admin JavaScript
 * ---------------------------------------------------------------------------*/
function stl_timeline_admin_head(){
	wp_enqueue_script('colorpicker');
	wp_enqueue_script('jquery-ui-core');
	wp_enqueue_script('jquery-ui-tabs');
	wp_enqueue_script('stl_timeline_admin_script');
}

/* -----------------------------------------------------------------------------
     Outputs necessary admin CSS (for post + options page)
 * ---------------------------------------------------------------------------*/
function stl_timeline_admin_css_head(){
	echo "<link rel=\"stylesheet\" type=\"text/css\" href=\"".WP_PLUGIN_URL."/wp-simile-timeline/options/timeline-admin.css\" />\n";
}

/* -----------------------------------------------------------------------------
     register admin options JavaScript
 * ---------------------------------------------------------------------------*/
function stl_stimeline_admin_init(){
	wp_register_script('stl_timeline_admin_script', WP_PLUGIN_URL . '/wp-simile-timeline/options/timeline-admin.js');
}

/* -----------------------------------------------------------------------------
 * stl_add_options_page
 * adds an admin page under Options > SIMILE Timeline
 * ---------------------------------------------------------------------------*/
function stl_add_options_page(){
	if (!empty($_POST['delete-action']) && $_POST['delete-action'] == 'purgedb'){
		stl_do_plugin_uninstall();
	}
	if (function_exists('add_options_page')) {
		$plugin_page = add_options_page('Timeline Options', 'SIMILE Timeline', 8, basename(__FILE__), 'stl_display_admin_page');
		// only add plugin related HTML head to option page
		add_action( 'admin_print_scripts', 'stl_timeline_admin_css_head' );
		add_action( 'admin_print_scripts-'. $plugin_page, 'stl_timeline_admin_head' );
		add_action('admin_init', 'stl_stimeline_admin_init');
	} 	
  
	// safety upgrade in case plugin wasn't inited by mechanism or user
	$version = get_option('stl_timeline_plugin_version');
	if(isset($version) && $version != STL_TIMELINE_PLUGIN_VERSION){
		stl_timeline_init_plugin();
	}

	// Update options when option page is submitted
	if (!empty($_POST['action']) && $_POST['action'] == 'update'){
		stl_update_options();
	}
}

/* -----------------------------------------------------------------------------
 * stl_timeline_post_add_custom_box
 * custom box hook for post interface adds custom option box
 * ---------------------------------------------------------------------------*/
function stl_timeline_post_add_custom_box() {
  if( function_exists( 'add_meta_box' )) {
    add_meta_box( 'stl-timeline-event-data', __( 'SIMILE Timeline', 'stl_timeline' ), 
                'stl_timeline_postdate_inner_custom_box', 'post', 'advanced' );
    add_meta_box( 'stl-timeline-event-data', __( 'SIMILE Timeline', 'stl_timeline' ), 
                'stl_timeline_postdate_inner_custom_box', 'page', 'advanced' );
   }
}

/* -----------------------------------------------------------------------------
 * stl_timeline_postdate_inner_custom_box
 * outputs content for custom option box in post/page interface
 * ---------------------------------------------------------------------------*/
function stl_timeline_postdate_inner_custom_box() {
	global $post, $action;
	// Use nonce for verification
	echo '<input type="hidden" name="st_timeline_nonce" id="st_timeline_nonce" value="' .  wp_create_nonce( plugin_basename(__FILE__) ) . '" />';
	echo '<table>
		<thead>
			<tr>
				<th>'.__('Event Start Date', true).'</th>
				<th>'.__('Event End Date', true).'</th>
			</tr>
		</thead>';
	echo '<tbody>';
	echo '<tr>';
	echo '<td>';
	echo '<h4>'.__('Start Date', true) . '</h4>';
	stl_timeline_touch_time(($action == 'edit'), 1, $post->stl_timeline_event_start, 'start');
	echo '</td>';
	echo '<td>';
	echo '<h4>'.__('End Date', true) . '</h4>';
	stl_timeline_touch_time(($action == 'edit'), 1, $post->stl_timeline_event_end, 'end');
	echo '</td>';
	echo '</tr>';
	echo '<tr>';
	echo '<td>';
	echo '<h4>'.__('Latest Start Date', true) . '</h4>';
	stl_timeline_touch_time(($action == 'edit'), 1, $post->stl_timeline_event_latest_start, 'latest_start');
	echo '</td>';
	echo '<td>';
	echo '<h4>'.__('Earliest End Date', true) . '</h4>';
	stl_timeline_touch_time(($action == 'edit'), 1, $post->stl_timeline_event_earliest_end, 'earliest_end');
	echo '</td>';
	echo '</tr>';
	echo '</tbody>
	</table>';
}

/* -----------------------------------------------------------------------------
 * stl_timeline_touch_time
 * output date selection elements
 * ---------------------------------------------------------------------------*/
function stl_timeline_touch_time($edit = 1, $for_post = 1, $type, $name) {
	global $month, $post, $comment;

	if ( $for_post )
		$edit = ( ('draft' == $post->post_status) && (!$type || '0000-00-00 00:00:00' == $type) ) ? false : true;
 
	echo '<legend>' .
		'<input type="checkbox" class="checkbox" name="stl_edit_timeline_'.$name.'_date" value="1" id="stl_timeline_'.$name.'_timestamp" /> <label for="stl_timeline_'.$name.'_timestamp">'. __('Edit timestamp','stl_timeline').'</label>' .
		'&nbsp;&nbsp;<input type="checkbox" class="checkbox" name="stl_reset_timeline_'.$name.'_date" value="1" id="stl_timeline_'.$name.'_timestamp_reset" /> <label for="stl_timeline_'.$name.'_timestamp_reset">'. __('Reset timestamp','stl_timeline').'</label>' .
				'</legend><br />';
	
	$stl_time_adj = time() + (get_settings('gmt_offset') * 3600);
	$post_date = ($for_post) ? $type : $comment->comment_date;
	
	$stl_jj = ($edit) ? adodb_date2('d', $post_date) : adodb_gmdate('d', $stl_time_adj);
	$stl_mm = ($edit) ? adodb_date2('m', $post_date) : adodb_gmdate('m', $stl_time_adj);
	$stl_aa = ($edit) ? adodb_date2('Y', $post_date) : adodb_gmdate('Y', $stl_time_adj);
	$stl_hh = ($edit) ? adodb_date2('H', $post_date) : adodb_gmdate('H', $stl_time_adj);
	$stl_mn = ($edit) ? adodb_date2('i', $post_date) : adodb_gmdate('i', $stl_time_adj);
	$stl_ss = ($edit) ? adodb_date2('s', $post_date) : adodb_gmdate('s', $stl_time_adj);
	
	if ( $edit && $post_date != '0000-00-00 00:00:00' ) {
		echo '<small>';
		_e('Existing timestamp');
		echo ": {$month[$stl_mm]} $stl_jj, $stl_aa @ $stl_hh:$stl_mn" . "</small>";
	}

	echo "<select name=\"stl_mm_".$name."\">\n";
	for ($i = 1; $i < 13; $i = $i +1) {
		echo "\t\t\t<option value=\"$i\"";
		if ($i == $stl_mm)
			echo " selected='selected'";
		if ($i < 10) {
			$stl_ii = "0".$i;
		} else {
			$stl_ii = "$i";
		}
		echo ">".$month["$stl_ii"]."</option>\n";
	}
?>
</select>
<input type="text" id="stl_jj_<?php echo $name; ?>" name="stl_jj_<?php echo $name; ?>" value="<?php echo $stl_jj; ?>" size="2" maxlength="2" />
<input type="text" id="stl_aa_<?php echo $name; ?>" name="stl_aa_<?php echo $name; ?>" value="<?php echo $stl_aa ?>" size="4" maxlength="5" /> @ 
<input type="text" id="stl_hh_<?php echo $name; ?>" name="stl_hh_<?php echo $name; ?>" value="<?php echo $stl_hh ?>" size="2" maxlength="2" /> : 
<input type="text" id="stl_mn_<?php echo $name; ?>" name="stl_mn_<?php echo $name; ?>" value="<?php echo $stl_mn ?>" size="2" maxlength="2" /> 
<input type="hidden" id="stl_ss_<?php echo $name; ?>" name="stl_ss_<?php echo $name; ?>" value="<?php echo $stl_ss ?>" size="2" maxlength="2" /> 


<?php
}

/** -----------------------------------------------------------------------------
 * stl_timeline_html_filter
 * Hybrid html filtering for XML-output
 * ---------------------------------------------------------------------------*/
function stl_timeline_html_filter($content){
	$content = htmlspecialchars($content);
	$content = ent2ncr($content);
	return $content;
}

/* Admin Menu actions
-----------------------*/

# add custom post boxes in post interface
add_action('admin_menu', 'stl_timeline_post_add_custom_box');
# save end date on post edit/save/publish
add_action('wp_insert_post', 'stl_timeline_update_eventstart_box');
add_action('wp_insert_post', 'stl_timeline_update_eventend_box');
# add options page
add_action('admin_menu', 'stl_add_options_page');

/* Init options
------------------------*/
# install new database tables & columns
register_activation_hook(__FILE__,'stl_timeline_init_plugin');
# ...for categories
add_action('add_category', 'stl_add_term_to_db');
#add_action('delete_category', 'stl_delete_cat_from_options');
add_action('delete_category', 'stl_delete_term_from_db');

/* Page filter (finally add the timeline)
---------------------------------------------*/
add_filter('wp_head', 'stl_headerJavaScriptCSS', 5);
add_shortcode('similetimeline', 'stl_output_timeline_markup');
?>